package edu.umd.cs.voting;

import java.awt.Color;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.util.HashMap;
import java.util.Iterator;

import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import org.jdom.DataConversionException;
import org.jdom.Element;

import edu.umd.cs.piccolo.PCamera;
import edu.umd.cs.piccolo.PCanvas;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.activities.PActivity;
import edu.umd.cs.piccolo.util.PAffineTransform;
import edu.umd.cs.piccolo.util.PNodeFilter;
import edu.umd.cs.piccolox.PFrame;
import edu.umd.cs.piccolox.event.PNotificationCenter;
import edu.umd.cs.voting.model.Ballot;
import edu.umd.cs.voting.view.BallotNode;
import edu.umd.cs.voting.view.ButtonNode;
import edu.umd.cs.voting.view.ButtonPageNode;
import edu.umd.cs.voting.view.CandidateNode;
import edu.umd.cs.voting.view.ItemNode;
import edu.umd.cs.voting.view.LowVisionMetalLookAndFeel;
import edu.umd.cs.voting.view.NavigationPanelNode;
import edu.umd.cs.voting.view.PageNode;
import edu.umd.cs.voting.view.PartiesComponent;

public class Controller extends PFrame {
	
	public static String FOCUS_CHANGED_NOTIFICATION = "FOCUS_CHANGED_NOTIFICATION";
	
	public static int NAVIGATION_PANEL_HEIGHT = 130;
	public static Controller CONTROLLER;
	public static long ANIMATION_TIME = 0;
	public static boolean IS_REVIEW_MODE = true;

	protected static String ballotFile = "./files/partiesBallot.xml";
	
	protected Ballot ballot;
	protected BallotNode ballotNode;
	protected PCanvas navigationCanvas;
	protected PNode focus;
		
	public void beforeInitialize() {
		new LowVisionMetalLookAndFeel(this);

		CONTROLLER = this;

		ballot = new Ballot(ballotFile);
		System.out.println("using file: " + ballotFile);
		
		Container contentPane = getContentPane();
		int width;
		int height;

		if (BallotNode.HIDE_CURSOR) {
			Image emptyCursorImage = new BufferedImage(1, 1, BufferedImage.TYPE_INT_ARGB);
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(emptyCursorImage, new Point(0,0), "what's this?");
			setCursor(cursor);
		}

		if (BallotNode.FULL_SCREEN_MODE) {
			setFullScreenMode(true);
			width = getWidth();
			height = getHeight();
		} else {
			setBounds(100, 100, (int) BallotNode.WIDTH, (int) BallotNode.HEIGHT);
			width = getWidth();
			height = getHeight() - 27;
		}				
		
		contentPane.setLayout(null);
		getCanvas().setBounds(0, 0, width, height - (NAVIGATION_PANEL_HEIGHT));
		navigationCanvas = new PCanvas();
		navigationCanvas.removeInputEventListener(navigationCanvas.getZoomEventHandler());
		navigationCanvas.removeInputEventListener(navigationCanvas.getPanEventHandler());
		contentPane.add(navigationCanvas);
		navigationCanvas.setBounds(0, height - (NAVIGATION_PANEL_HEIGHT), width, NAVIGATION_PANEL_HEIGHT);		
		validate();
		
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				showPassword();				
			}
		});
	}

	public void initialize() {
		ballotNode = new BallotNode(ballot, this);
		getCanvas().getLayer().addChild(ballotNode);

		PartiesComponent partiesComponent = ballot.getPartiesComponent();		
		if (partiesComponent != null) {
			JComponent canvas = getCanvas();
			partiesComponent.setCursor(getCursor());
			canvas.add(partiesComponent);
			partiesComponent.setLocation(10, canvas.getHeight() - (partiesComponent.getHeight() + 10));
			partiesComponent.invalidate();
			partiesComponent.revalidate();
		}

		ANIMATION_TIME = 0;
		setFocusNode(ballotNode);
		ANIMATION_TIME = 500;
	}
	
	public Ballot getBallot() {
		return ballot;
	}
	
	public BallotNode getBallotNode() {
		return ballotNode;
	}
	
	public PNode getFocusNode() {
		return focus;
	}

	public void setFocusNode(PNode node) {
		final PNode oldFocus = focus;
		
		setReviewMode(false);
		
		if (node == focus) {
			Toolkit.getDefaultToolkit().beep();
			return;
		}
		
		if (focus instanceof ItemNode) {
			((ItemNode)focus).setHasFocus(false);
		} else if (focus instanceof PageNode) {
			((PageNode)focus).setHasFocus(false);			
		}

		focus = node;

		ballotNode.setFullFaded(true);
		
		PCamera camera = getCanvas().getCamera();
		PAffineTransform originalTransform = camera.getViewTransform();
			
		if (node instanceof BallotNode) {
			camera.animateViewToCenterBounds(node.getGlobalFullBounds(), true, 0);
			ballotNode.setFullFaded(false);
		} else if (node instanceof PageNode) {
			((PageNode)focus).setHasFocus(true);
			camera.animateViewToCenterBounds(node.getGlobalFullBounds(), true, 0);			
			((PageNode)node).setFullFaded(false);
		} else if (node instanceof ItemNode) {
			if (node.getParent() != camera) {
				camera.setViewScale(1);
				camera.animateViewToCenterBounds(node.getGlobalFullBounds(), false, 0);
	
				((PageNode)node.getParent()).setFaded(false);
				((ItemNode)focus).setHasFocus(true);
			}
		}
		
		if (camera.getViewScale() > 1) {
			camera.setViewScale(1);
			camera.animateViewToCenterBounds(node.getGlobalFullBounds(), false, 0);
		}
		
		PAffineTransform destinationTransform = camera.getViewTransform();
		camera.setViewTransform(originalTransform);
		PActivity animationActivity = camera.animateViewToTransform(destinationTransform, ANIMATION_TIME);

		if (animationActivity != null) {
			animationActivity.setDelegate(new PActivity.PActivityDelegate(){
				public void activityStarted(PActivity activity) {
					if (oldFocus instanceof BallotNode) {
						PartiesComponent partiesComponent = ballot.getPartiesComponent();		
						if (partiesComponent != null) {
							partiesComponent.setVisible(false);
						}
					}
				}
				
				public void activityStepped(PActivity activity) {}
				public void activityFinished(PActivity activity) {
					if (focus instanceof BallotNode) {
						PartiesComponent partiesComponent = ballot.getPartiesComponent();		
						if (partiesComponent != null) {
							partiesComponent.setVisible(true);
						}
					}
				}
			});
		}
				
		PNotificationCenter.defaultCenter().postNotification(FOCUS_CHANGED_NOTIFICATION, this);
	}

	public PCanvas getNavigationCanvas() {
		return navigationCanvas;
	}
	
	public void submitVote() {
		ballot.submit();
		ballotNode.clearBallot();
		setFocusNode(ballotNode);
		showPassword();
	}

	public void showPassword() {
		String password = null;
		
		while (password == null || !password.equalsIgnoreCase("123")) {
			password = (String)JOptionPane.showInputDialog(
	                this,
	                "Ballot Successfully Cast!\nEnter Code For New Ballot",
	                "Voter Verification",
	                JOptionPane.QUESTION_MESSAGE,
	                null,
	                null,
	                null);
			
			if (password == null || !password.equalsIgnoreCase("123")) {
				Toolkit.getDefaultToolkit().beep();
			}
		}
	}
	
	public void setReviewMode(boolean reviewMode) {
		if (IS_REVIEW_MODE != reviewMode) {
			IS_REVIEW_MODE = reviewMode;

			if (IS_REVIEW_MODE) {
				NavigationPanelNode.REVIEW_AND_SUBMIT_BUTTON.setLable("Cast Ballot");
				NavigationPanelNode.REVIEW_AND_SUBMIT_BUTTON.setPaint(ButtonNode.REVIEW_AND_SUBMIT_BUTTON_REVIEW_MODE_COLOR);
			} else {
				NavigationPanelNode.REVIEW_AND_SUBMIT_BUTTON.setLable("Review and Cast Ballot");
				NavigationPanelNode.REVIEW_AND_SUBMIT_BUTTON.setPaint(ButtonNode.REVIEW_AND_SUBMIT_BUTTON_COLOR);
			}
			
			PNodeFilter filter = new PNodeFilter() {
				public boolean accept(PNode aNode) {
					return aNode instanceof ItemNode;
				}
				public boolean acceptChildrenOf(PNode aNode) {
					return !(aNode instanceof ItemNode);
				}
			};
			
			Iterator i = getBallotNode().getAllNodes(filter, null).iterator();
			while (i.hasNext()) { 
				ItemNode each = (ItemNode) i.next();
				each.setHasFocus(!each.getHasFocus());
				each.setHasFocus(!each.getHasFocus());
			}
		}
	}
	
	protected static HashMap ACTIVITIES = new HashMap();
	
	public static void schedualNewActivityFor(PNode node, PActivity activity) {
		PActivity a = (PActivity) ACTIVITIES.get(node);
		if (a != null) {
			a.terminate();
		}
		ACTIVITIES.put(node, activity);
	}

	public static void loadStyleInformation(Element style) {
		BallotNode.loadStyleInformation(style.getChild("ballot-style")); 
		PageNode.loadStyleInformation(style.getChild("page-style"));
		ItemNode.loadStyleInformation(style.getChild("item-style"));
		CandidateNode.loadStyleInformation(style.getChild("candidate-style"));
		ButtonNode.loadStyleInformation(style.getChild("button-style"));
		ButtonNode.loadNextPreviousStyleInformation(style.getChild("next-previous-button-style"));
//		ButtonNode.loadSubmitStyleInformation(style.getChild("submit-button-style"));
		ButtonNode.loadReviewAndSubmitStyleInformation(style.getChild("review-and-submit-button-style"));
		ButtonPageNode.loadStyleInformation(style.getChild("button-page-style"));
	}

	public static Color readColorStyle(String color) {
		String[] c = color.split(",");
		return new Color(Float.parseFloat(c[0]), Float.parseFloat(c[1]), Float.parseFloat(c[2]));
	}

	public static Font readFontStyle(Element font) {
		String styleString = font.getAttributeValue("style");
		int style = Font.PLAIN;
		
		if (styleString.equals("bold")) {
			style = Font.BOLD;
		} else if (styleString.equals("italic")) {
			style = Font.ITALIC;
		} else {
			style = Font.PLAIN;
		}
		
		try {
			return new Font(font.getAttributeValue("font"), style, font.getAttribute("size").getIntValue());
		} catch (DataConversionException e) {
			e.printStackTrace();
		}
		
		return null;
	}

	public static void main(String[] args) {
		if (args.length > 0) {
			System.out.println(args[0]);
			Controller.ballotFile = args[0];
		}
		new Controller();			
	}
}
