package edu.umd.cs.voting.view;

import java.awt.Color;
import java.awt.Font;

import org.jdom.Element;

import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.activities.PActivity;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolo.util.PBounds;
import edu.umd.cs.piccolox.event.PNotification;
import edu.umd.cs.piccolox.event.PNotificationCenter;
import edu.umd.cs.voting.Controller;
import edu.umd.cs.voting.model.Candidate;
import edu.umd.cs.voting.model.Item;


public class ButtonPageNode extends ButtonNode {

	protected static Font COMPLETED_COUNT_FONT = new Font("Helvetica", Font.BOLD, 36);
	protected static Color PROGRESS_BAR_COLOR = Color.GREEN;

	protected int questionCount;
	protected int completedCount;
	protected PNode progressBarNode;
	protected PText completedCountText;
	
	public ButtonPageNode(PageNode page) {
		super((String)page.getPage().getShortTitle(), page);
		
		PNotificationCenter.defaultCenter().addListener(this, "candidateSelectedStateChanged", Candidate.SELECTED_STATE_CHANGED, null);

		progressBarNode = new PNode();
		progressBarNode.setPaint(PROGRESS_BAR_COLOR);
		progressBarNode.setBounds(0, getY() + getHeight(), getWidth(), 0);
		progressBarNode.setPickable(false);
		addChild(progressBarNode);
		progressBarNode.moveToBack();
		
		completedCountText = new PText();
		completedCountText.setFont(COMPLETED_COUNT_FONT);
		completedCountText.setPickable(false);
		addChild(completedCountText);

		calculatePercentComplete();
		if (page.page.getItems().size() > 0) {
			setPaint(page.getPaint());
		}
	}
		
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);

		if (isEnabled) {
			completedCountText.setTextPaint(Color.BLACK);
		} else {
			completedCountText.setTextPaint(DISABLED_TEXT_COLOR);
		}
	}
		
	public void positionLabel() {
		if (completedCountText != null) {
			completedCountText.centerBoundsOnPoint(getBoundsReference().getCenterX(), getBoundsReference().getCenterY() - 15);
			buttonLabel.centerBoundsOnPoint(getBoundsReference().getCenterX(), getBoundsReference().getCenterY() + 15);
		} else {
			super.positionLabel();
		}
	}
		
	public void calculatePercentComplete() {
		int totalItems = 0;
		int completedItems = 0;
		questionCount = 0;
		completedCount = 0;
		
		totalItems = ((PageNode)targetNode).getPage().getItems().size();
		for (int i = 0; i < totalItems; i++) {
			Item each = (Item) ((PageNode)targetNode).getPage().getItems().get(i);
			questionCount += each.getMaxSelectedCandidates();
			completedCount += each.getNumCurrentlySelectedCandidates();
			if (each.getMaxSelectedCandidates() == each.getNumCurrentlySelectedCandidates()) {
				completedItems++;
			}	
		}
		
		if (questionCount == 0) return;
			
		PBounds bounds = getBounds();
//		double completedPercent = (double) completedCount / (double) questionCount;
		double completedPercent = (double) completedItems / (double) totalItems;
			
		PActivity a = progressBarNode.animateToBounds(
			bounds.getX(),
			bounds.getY() + (bounds.getHeight() - (bounds.getHeight() * completedPercent)),
			bounds.getWidth(),
			bounds.getHeight() * completedPercent,
			Controller.ANIMATION_TIME);

		Controller.schedualNewActivityFor(progressBarNode, a);

		//		completedCountText.setText(completedCount + "/" + questionCount);
		completedCountText.setText(completedItems + "/" + totalItems);
		positionLabel();
	}
	
	public void candidateSelectedStateChanged(PNotification notification) {
		if (!CandidateNode.IS_ANIMATING) {
			calculatePercentComplete();
			invalidatePaint();
		}
	}
	
	public static void loadStyleInformation(Element style) {
		ButtonPageNode.COMPLETED_COUNT_FONT = Controller.readFontStyle(style.getChild("completedCount"));
		ButtonPageNode.PROGRESS_BAR_COLOR = Controller.readColorStyle(style.getAttributeValue("progresscolor"));
	}
}
