package edu.umd.cs.voting.view;

/*
 * LowVisionMetalLookAndFeel.java
 */
import java.awt.Component;
import java.awt.Font;

import javax.swing.SwingUtilities;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.border.Border;
import javax.swing.border.LineBorder;
import javax.swing.plaf.BorderUIResource;
import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.FontUIResource;
import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalIconFactory;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.plaf.metal.MetalTheme;

/**
 * Implements a Metal look and feel that automatically loads a low-vision
 * theme. The low-vision theme has large fonts and high-contrast colors
 *
 * @author Steve Wilson
 * @author Michael C. Albers
 * @author Jeff Dunn
 */
public class LowVisionMetalLookAndFeel extends MetalLookAndFeel {

	public LowVisionMetalLookAndFeel(Component aComponent) {
	super();
		MetalTheme theme = new LowVisionMetalTheme();
		setCurrentTheme(theme);
		
		try {
			UIManager.setLookAndFeel(new MetalLookAndFeel());
		} catch (UnsupportedLookAndFeelException e) {
			e.printStackTrace();
		}
		
		SwingUtilities.updateComponentTreeUI(aComponent);
	}

	protected class LowVisionMetalTheme extends DefaultMetalTheme {

		public String getName() { return "Low Vision"; }

	// Colors
		private final ColorUIResource primary1 = new ColorUIResource(0, 0, 0);
		private final ColorUIResource primary2 = new ColorUIResource(204, 204, 204);
		private final ColorUIResource primary3 = new ColorUIResource(255, 255, 255);
		private final ColorUIResource primaryHighlight = new ColorUIResource(102,102,102);

		private final ColorUIResource secondary2 = new ColorUIResource(204, 204, 204);
		private final ColorUIResource secondary3 = new ColorUIResource(255, 255, 255);
		private final ColorUIResource controlHighlight = new ColorUIResource(102,102,102);

		protected ColorUIResource getPrimary1() { return primary1; }
		protected ColorUIResource getPrimary2() { return primary2; }
		protected ColorUIResource getPrimary3() { return primary3; }
		public ColorUIResource getPrimaryControlHighlight() { return primaryHighlight;}

		protected ColorUIResource getSecondary2() { return secondary2; }
		protected ColorUIResource getSecondary3() { return secondary3; }
		public ColorUIResource getControlHighlight() { return super.getSecondary3(); }

		public ColorUIResource getFocusColor() { return getBlack(); }

		public ColorUIResource getTextHighlightColor() { return getBlack(); }
		public ColorUIResource getHighlightedTextColor() { return getWhite(); }

		public ColorUIResource getMenuSelectedBackground() { return getBlack(); }
		public ColorUIResource getMenuSelectedForeground() { return getWhite(); }
		public ColorUIResource getAcceleratorForeground() { return getBlack(); }
		public ColorUIResource getAcceleratorSelectedForeground() { return getWhite(); }

	// Fonts
		private final FontUIResource controlFont = new FontUIResource("Dialog", Font.BOLD, 24);
		private final FontUIResource systemFont = new FontUIResource("Dialog", Font.PLAIN, 24);
		private final FontUIResource windowTitleFont = new FontUIResource("Dialog", Font.BOLD, 24);
		private final FontUIResource userFont = new FontUIResource("SansSerif", Font.PLAIN, 24);
		private final FontUIResource smallFont = new FontUIResource("Dialog", Font.PLAIN, 20);

		public FontUIResource getControlTextFont() { return controlFont;}
		public FontUIResource getSystemTextFont() { return systemFont;}
		public FontUIResource getUserTextFont() { return userFont;}
		public FontUIResource getMenuTextFont() { return controlFont;}
		public FontUIResource getWindowTitleFont() { return windowTitleFont;}
		public FontUIResource getSubTextFont() { return smallFont;}

		public void addCustomEntriesToTable(UIDefaults table) {
			 super.addCustomEntriesToTable(table);

			 final int internalFrameIconSize = 30;
			 table.put("InternalFrame.closeIcon", MetalIconFactory.getInternalFrameCloseIcon(internalFrameIconSize));
			 table.put("InternalFrame.maximizeIcon", MetalIconFactory.getInternalFrameMaximizeIcon(internalFrameIconSize));
			 table.put("InternalFrame.iconifyIcon", MetalIconFactory.getInternalFrameMinimizeIcon(internalFrameIconSize));
			 table.put("InternalFrame.minimizeIcon", MetalIconFactory.getInternalFrameAltMaximizeIcon(internalFrameIconSize));


		Border blackLineBorder = new BorderUIResource(new LineBorder(getBlack()));
		Border whiteLineBorder = new BorderUIResource(new LineBorder(getWhite()));
		Border textBorder = blackLineBorder;

			table.put( "ToolTip.border", blackLineBorder);
		table.put( "TitledBorder.border", blackLineBorder);
			table.put( "Table.focusCellHighlightBorder", whiteLineBorder);
			table.put( "Table.focusCellForeground", getWhite());
			table.put( "TextField.border", textBorder);
			table.put( "PasswordField.border", textBorder);
			table.put( "TextArea.border", textBorder);
			table.put( "TextPane.border", textBorder);
			table.put( "ScrollPane.border", blackLineBorder);
			table.put( "ScrollBar.width", new Integer(25) );
		} // addCustomEntriesToTable()
	} // class LowVisionMetalTheme
} // class LowVisionMetalLookAndFeel()
